@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -958,6 +958,11 @@ public:
         return state;
     };
 
+    VmState get_prev_state() const
+    {
+        return prev_state;
+    };
+
     /**
      *  Returns the VM state (life-cycle Manager)
      *    @return the VM state
@@ -967,6 +972,11 @@ public:
         return lcm_state;
     };
 
+    LcmState get_prev_lcm_state() const
+    {
+        return prev_lcm_state;
+    };
+
     /**
      *  Sets VM state
      *    @param s state
@@ -985,6 +995,24 @@ public:
         lcm_state = s;
     };
 
+    /**
+     *  Sets the previous state to the current one
+     */
+    void set_prev_state()
+    {
+        prev_state     = state;
+        prev_lcm_state = lcm_state;
+    };
+
+    /**
+     *  Test if the VM has changed state since last time prev state was set
+     *    @return true if VM changed state
+     */
+    bool has_changed_state()
+    {
+        return (prev_lcm_state != lcm_state || prev_state != state);
+    }
+
     /**
      *  Sets the re-scheduling flag
      *    @param set or unset the re-schedule flag
@@ -1055,12 +1083,6 @@ public:
     // ------------------------------------------------------------------------
     // Network Leases & Disk Images
     // ------------------------------------------------------------------------
-    /**
-     *  Get all network leases for this Virtual Machine
-     *  @return 0 if success
-     */
-    int get_network_leases(string &error_str);
-
     /**
      *  Releases all network leases taken by this Virtual Machine
      */
@@ -1076,13 +1098,6 @@ public:
      */
     static int release_network_leases(VectorAttribute const * nic, int vmid);
 
-    /**
-     *  Get all disk images for this Virtual Machine
-     *  @param error_str Returns the error reason, if any
-     *  @return 0 if success
-     */
-    int get_disk_images(string &error_str);
-
     /**
      *  Releases all disk images taken by this Virtual Machine
      */
@@ -1103,6 +1118,12 @@ public:
      */
     static long long get_volatile_disk_size(Template * tmpl);
 
+    /**
+     * Returns a set of the security group IDs in use in this VM
+     * @param sgs a set of security group IDs
+     */
+    void get_security_groups(set<int>& sgs) const;
+
     // ------------------------------------------------------------------------
     // Context related functions
     // ------------------------------------------------------------------------
@@ -1211,7 +1232,7 @@ public:
      *    @param image_id id of the image being saved
      *    @return IMAGE_ID on success, -1 otherwise
      */
-    int get_disk_hot_info(int& image_id, int& disk_id, string& source);
+    //int get_disk_hot_info(int& image_id, int& disk_id, string& source);
 
     /**
      * Generate a DISK attribute to be attached to the VM.
@@ -1293,7 +1314,11 @@ public:
      * Setups the new NIC attribute to be attached to the VM.
      *
      * @param vm_id Id of the VM where this nic will be attached
+     * @param vm_sgs the securty group ids already present in the VM
      * @param new_nic New NIC vector attribute, obtained from get_attach_nic_info
+     * @param rules Security Group rules will be added at the end of this
+     * vector. If not used, the VectorAttributes must be freed by the calling
+     * method
      * @param max_nic_id Max NIC/NIC_ID of the VM
      * @param uid of the VM owner
      * @param error_str error reason, if any
@@ -1301,7 +1326,9 @@ public:
      */
     static int set_up_attach_nic(
                             int                      vm_id,
+                            set<int>&                vm_sgs,
                             VectorAttribute *        new_nic,
+                            vector<VectorAttribute*> &rules,
                             int                      max_nic_id,
                             int                      uid,
                             string&                  error_str);
@@ -1322,13 +1349,9 @@ public:
      *  Adds a new NIC to the virtual machine template. The NIC should be
      *  generated by the build_attach_nic
      *    @param new_nic must be allocated in the heap
+     *    @param rules Security Group rules obtained from set_up_attach_nic
      */
-    void set_attach_nic(VectorAttribute * new_nic)
-    {
-        new_nic->replace("ATTACH", "YES");
-
-        obj_template->set(new_nic);
-    }
+    void set_attach_nic(VectorAttribute * new_nic, vector<VectorAttribute*> &rules);
 
     /**
      *  Sets the attach attribute to the given NIC
@@ -1423,11 +1446,21 @@ private:
      */
     VmState     state;
 
+    /**
+     *  Previous state og the virtual machine, to trigger state hooks
+     */
+    VmState     prev_state;
+
     /**
      *  The state of the virtual machine (in the Life-cycle Manager).
      */
     LcmState    lcm_state;
 
+    /**
+     *  Previous state og the virtual machine, to trigger state hooks
+     */
+    LcmState    prev_lcm_state;
+
     /**
      *  Marks the VM as to be re-scheduled
      */
@@ -1517,10 +1550,12 @@ private:
         ostringstream oss_vm(VirtualMachine::db_bootstrap);
         ostringstream oss_monit(VirtualMachine::monit_db_bootstrap);
         ostringstream oss_hist(History::db_bootstrap);
+        ostringstream oss_showback(VirtualMachine::showback_db_bootstrap);
 
         rc =  db->exec(oss_vm);
         rc += db->exec(oss_monit);
         rc += db->exec(oss_hist);
+        rc += db->exec(oss_showback);
 
         return rc;
     };
@@ -1681,6 +1716,50 @@ private:
      */
     void merge_nic_defaults(VectorAttribute* nic);
 
+    // -------------------------------------------------------------------------
+    // NIC & DISK Management Helpers
+    // -------------------------------------------------------------------------
+    /**
+     *  Get all network leases for this Virtual Machine
+     *  @return 0 if success
+     */
+    int get_network_leases(string &error_str);
+
+    /**
+     * Acquires the security groups of this NIC
+     *
+     * @param vm_id Virtual Machine oid
+     * @param sgs security group ID set
+     * @param rules Security Group rules will be added at the end of this vector
+     */
+    static void get_security_group_rules(int vm_id, set<int>& sgs,
+        vector<VectorAttribute*> &rules);
+
+    /**
+     * Releases the security groups of this NIC
+     *
+     * @param vm_id Virtual Machine oid
+     * @param nic NIC to release the security groups
+     * @return 0 on success, -1 otherwise
+     */
+    static void release_security_groups(int vm_id, VectorAttribute const * nic);
+
+    /**
+     * Returns a set of the security group IDs of this NIC
+     * @param nic NIC to get the security groups from
+     * @param sgs a set of security group IDs
+     */
+    static void get_security_groups(VectorAttribute const * nic, set<int>& sgs)
+    {
+        one_util::split_unique(nic->vector_value("SECURITY_GROUPS"), ',', sgs);
+    }
+
+    /**
+     *  Get all disk images for this Virtual Machine
+     *  @param error_str Returns the error reason, if any
+     *  @return 0 if success
+     */
+    int get_disk_images(string &error_str);
 
 protected:
 
@@ -1714,6 +1793,12 @@ protected:
 
     static const char * monit_db_bootstrap;
 
+    static const char * showback_table;
+
+    static const char * showback_db_names;
+
+    static const char * showback_db_bootstrap;
+
     /**
      *  Reads the Virtual Machine (identified with its OID) from the database.
      *    @param db pointer to the db
