@@ -1,7 +1,7 @@
 #!/bin/bash
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -117,6 +117,7 @@ if [ -z "$ROOT" ] ; then
     SHARE_LOCATION="/usr/share/one"
     MAN_LOCATION="/usr/share/man/man1"
     VM_LOCATION="/var/lib/one/vms"
+    DOCS_LOCATION="/usr/share/docs/one"
 
     if [ "$CLIENT" = "yes" ]; then
         MAKE_DIRS="$BIN_LOCATION $LIB_LOCATION $ETC_LOCATION"
@@ -147,7 +148,7 @@ if [ -z "$ROOT" ] ; then
         CHOWN_DIRS=""
     else
         MAKE_DIRS="$BIN_LOCATION $LIB_LOCATION $ETC_LOCATION $VAR_LOCATION \
-                   $INCLUDE_LOCATION $SHARE_LOCATION \
+                   $INCLUDE_LOCATION $SHARE_LOCATION $DOCS_LOCATION \
                    $LOG_LOCATION $RUN_LOCATION $LOCK_LOCATION \
                    $SYSTEM_DS_LOCATION $DEFAULT_DS_LOCATION $MAN_LOCATION \
                    $VM_LOCATION $ONEGATE_LOCATION $ONEFLOW_LOCATION"
@@ -172,6 +173,7 @@ else
     SHARE_LOCATION="$ROOT/share"
     MAN_LOCATION="$ROOT/share/man/man1"
     VM_LOCATION="$VAR_LOCATION/vms"
+    DOCS_LOCATION="$ROOT/share/docs"
 
     if [ "$CLIENT" = "yes" ]; then
         MAKE_DIRS="$BIN_LOCATION $LIB_LOCATION $ETC_LOCATION"
@@ -195,7 +197,7 @@ else
     else
         MAKE_DIRS="$BIN_LOCATION $LIB_LOCATION $ETC_LOCATION $VAR_LOCATION \
                    $INCLUDE_LOCATION $SHARE_LOCATION $SYSTEM_DS_LOCATION \
-                   $DEFAULT_DS_LOCATION $MAN_LOCATION \
+                   $DEFAULT_DS_LOCATION $MAN_LOCATION $DOCS_LOCATION \
                    $VM_LOCATION $ONEGATE_LOCATION $ONEFLOW_LOCATION"
 
         DELETE_DIRS="$MAKE_DIRS"
@@ -262,6 +264,7 @@ VAR_DIRS="$VAR_LOCATION/remotes \
           $VAR_LOCATION/remotes/vmm/az \
           $VAR_LOCATION/remotes/vnm \
           $VAR_LOCATION/remotes/vnm/802.1Q \
+          $VAR_LOCATION/remotes/vnm/vxlan \
           $VAR_LOCATION/remotes/vnm/dummy \
           $VAR_LOCATION/remotes/vnm/ebtables \
           $VAR_LOCATION/remotes/vnm/fw \
@@ -326,6 +329,7 @@ SUNSTONE_DIRS="$SUNSTONE_LOCATION/routes \
                $SUNSTONE_LOCATION/public/locale/zh_CN \
                $SUNSTONE_LOCATION/public/vendor \
                $SUNSTONE_LOCATION/public/vendor/crypto-js \
+               $SUNSTONE_LOCATION/public/vendor/spice \
                $SUNSTONE_LOCATION/public/vendor/noVNC \
                $SUNSTONE_LOCATION/public/vendor/noVNC/web-socket-js \
                $SUNSTONE_LOCATION/public/vendor/4.0 \
@@ -448,6 +452,7 @@ INSTALL_FILES=(
     DATASTORE_DRIVER_DEV_SCRIPTS:$VAR_LOCATION/remotes/datastore/dev
     NETWORK_FILES:$VAR_LOCATION/remotes/vnm
     NETWORK_8021Q_FILES:$VAR_LOCATION/remotes/vnm/802.1Q
+    NETWORK_VXLAN_FILES:$VAR_LOCATION/remotes/vnm/vxlan
     NETWORK_DUMMY_FILES:$VAR_LOCATION/remotes/vnm/dummy
     NETWORK_EBTABLES_FILES:$VAR_LOCATION/remotes/vnm/ebtables
     NETWORK_FW_FILES:$VAR_LOCATION/remotes/vnm/fw
@@ -466,6 +471,7 @@ INSTALL_FILES=(
     MARKET_LIB_FILES:$LIB_LOCATION/ruby/cloud/marketplace
     MARKET_BIN_FILES:$BIN_LOCATION
     MAN_FILES:$MAN_LOCATION
+    DOCS_FILES:$DOCS_LOCATION
     CLI_LIB_FILES:$LIB_LOCATION/ruby/cli
     ONE_CLI_LIB_FILES:$LIB_LOCATION/ruby/cli/one_helper
     RBVMOMI_VENDOR_RUBY_FILES:$LIB_LOCATION/ruby/vendors/rbvmomi
@@ -507,6 +513,7 @@ INSTALL_SUNSTONE_FILES=(
     SUNSTONE_ROUTES_FILES:$SUNSTONE_LOCATION/routes
     SUNSTONE_PUBLIC_CSS_FILES:$SUNSTONE_LOCATION/public/css
     SUNSTONE_PUBLIC_VENDOR_CRYPTOJS:$SUNSTONE_LOCATION/public/vendor/crypto-js
+    SUNSTONE_PUBLIC_VENDOR_SPICE:$SUNSTONE_LOCATION/public/vendor/spice
     SUNSTONE_PUBLIC_VENDOR_NOVNC:$SUNSTONE_LOCATION/public/vendor/noVNC
     SUNSTONE_PUBLIC_VENDOR_NOVNC_WEBSOCKET:$SUNSTONE_LOCATION/public/vendor/noVNC/web-socket-js
     SUNSTONE_PUBLIC_NEW_VENDOR_FLOT:$SUNSTONE_LOCATION/public/vendor/4.0/flot
@@ -591,6 +598,7 @@ BIN_FILES="src/nebula/oned \
            src/scheduler/src/sched/mm_sched \
            src/cli/onevm \
            src/cli/oneacct \
+           src/cli/oneshowback \
            src/cli/onehost \
            src/cli/onevnet \
            src/cli/oneuser \
@@ -603,6 +611,8 @@ BIN_FILES="src/nebula/oned \
            src/cli/onezone \
            src/cli/oneflow \
            src/cli/oneflow-template \
+           src/cli/onesecgroup \
+           src/cli/onevdc \
            src/cli/onevcenter \
            src/onedb/onedb \
            src/mad/utils/tty_expect \
@@ -928,15 +938,29 @@ AUTH_PLAIN_FILES="src/authm_mad/remotes/plain/authenticate"
 # Virtual Network Manager drivers to be installed under $REMOTES_LOCATION/vnm
 #-------------------------------------------------------------------------------
 
-NETWORK_FILES="src/vnm_mad/remotes/OpenNebulaNetwork.rb \
+NETWORK_FILES="src/vnm_mad/remotes/lib/vnm_driver.rb \
+               src/vnm_mad/remotes/lib/vnmmad.rb \
                src/vnm_mad/remotes/OpenNebulaNetwork.conf \
-               src/vnm_mad/remotes/Firewall.rb \
-               src/vnm_mad/remotes/OpenNebulaNic.rb"
+               src/vnm_mad/remotes/lib/fw_driver.rb \
+               src/vnm_mad/remotes/lib/sg_driver.rb \
+               src/vnm_mad/remotes/lib/address.rb \
+               src/vnm_mad/remotes/lib/command.rb \
+               src/vnm_mad/remotes/lib/vm.rb \
+               src/vnm_mad/remotes/lib/vlan.rb \
+               src/vnm_mad/remotes/lib/security_groups.rb \
+               src/vnm_mad/remotes/lib/security_groups_iptables.rb \
+               src/vnm_mad/remotes/lib/nic.rb"
 
 NETWORK_8021Q_FILES="src/vnm_mad/remotes/802.1Q/clean \
                     src/vnm_mad/remotes/802.1Q/post \
                     src/vnm_mad/remotes/802.1Q/pre \
-                    src/vnm_mad/remotes/802.1Q/HostManaged.rb"
+                    src/vnm_mad/remotes/802.1Q/vlan_tag_driver.rb"
+
+NETWORK_VXLAN_FILES="src/vnm_mad/remotes/vxlan/clean \
+                    src/vnm_mad/remotes/vxlan/post \
+                    src/vnm_mad/remotes/vxlan/pre \
+                    src/vnm_mad/remotes/vxlan/vxlan_driver.rb"
+
 
 NETWORK_DUMMY_FILES="src/vnm_mad/remotes/dummy/clean \
                     src/vnm_mad/remotes/dummy/post \
@@ -1177,10 +1201,13 @@ ONEDB_SHARED_MIGRATOR_FILES="src/onedb/shared/2.0_to_2.9.80.rb \
                              src/onedb/shared/4.3.90_to_4.4.0.rb \
                              src/onedb/shared/4.4.0_to_4.4.1.rb \
                              src/onedb/shared/4.4.1_to_4.5.80.rb\
-                             src/onedb/shared/4.5.80_to_4.6.0.rb"
+                             src/onedb/shared/4.5.80_to_4.6.0.rb \
+                             src/onedb/shared/4.6.0_to_4.11.80.rb"
 
 ONEDB_LOCAL_MIGRATOR_FILES="src/onedb/local/4.5.80_to_4.7.80.rb \
-                            src/onedb/local/4.7.80_to_4.9.80.rb"
+                            src/onedb/local/4.7.80_to_4.9.80.rb \
+                            src/onedb/local/4.9.80_to_4.10.3.rb \
+                            src/onedb/local/4.10.3_to_4.11.80.rb"
 
 #-------------------------------------------------------------------------------
 # Configuration files for OpenNebula, to be installed under $ETC_LOCATION
@@ -1212,7 +1239,8 @@ VMM_EXEC_ETC_FILES="src/vmm_mad/exec/vmm_execrc \
                   src/vmm_mad/exec/vmm_exec_kvm.conf \
                   src/vmm_mad/exec/vmm_exec_xen3.conf \
                   src/vmm_mad/exec/vmm_exec_xen4.conf \
-                  src/vmm_mad/exec/vmm_exec_vmware.conf"
+                  src/vmm_mad/exec/vmm_exec_vmware.conf \
+                  src/vmm_mad/exec/vmm_exec_vcenter.conf"
 
 #-------------------------------------------------------------------------------
 # Hook Manager driver config. files, to be installed under $ETC_LOCATION/hm
@@ -1279,15 +1307,18 @@ RUBY_OPENNEBULA_LIB_FILES="src/oca/ruby/opennebula/acl_pool.rb \
                             src/oca/ruby/opennebula/host.rb \
                             src/oca/ruby/opennebula/image_pool.rb \
                             src/oca/ruby/opennebula/image.rb \
+                            src/oca/ruby/opennebula/oneflow_client.rb \
                             src/oca/ruby/opennebula/pool_element.rb \
                             src/oca/ruby/opennebula/pool.rb \
+                            src/oca/ruby/opennebula/security_group_pool.rb \
+                            src/oca/ruby/opennebula/security_group.rb \
                             src/oca/ruby/opennebula/system.rb \
                             src/oca/ruby/opennebula/template_pool.rb \
                             src/oca/ruby/opennebula/template.rb \
                             src/oca/ruby/opennebula/user_pool.rb \
                             src/oca/ruby/opennebula/user.rb \
-                            src/oca/ruby/opennebula/zone_pool.rb \
-                            src/oca/ruby/opennebula/zone.rb \
+                            src/oca/ruby/opennebula/vdc_pool.rb \
+                            src/oca/ruby/opennebula/vdc.rb \
                             src/oca/ruby/opennebula/virtual_machine_pool.rb \
                             src/oca/ruby/opennebula/virtual_machine.rb \
                             src/oca/ruby/opennebula/virtual_network_pool.rb \
@@ -1295,7 +1326,8 @@ RUBY_OPENNEBULA_LIB_FILES="src/oca/ruby/opennebula/acl_pool.rb \
                             src/oca/ruby/opennebula/xml_element.rb \
                             src/oca/ruby/opennebula/xml_pool.rb \
                             src/oca/ruby/opennebula/xml_utils.rb \
-                            src/oca/ruby/opennebula/oneflow_client.rb"
+                            src/oca/ruby/opennebula/zone_pool.rb \
+                            src/oca/ruby/opennebula/zone.rb"
 
 #-------------------------------------------------------------------------------
 # Common Cloud Files
@@ -1443,7 +1475,9 @@ ONE_CLI_LIB_FILES="src/cli/one_helper/onegroup_helper.rb \
                    src/cli/one_helper/onedatastore_helper.rb \
                    src/cli/one_helper/onecluster_helper.rb \
                    src/cli/one_helper/onezone_helper.rb \
-                   src/cli/one_helper/oneacct_helper.rb"
+                   src/cli/one_helper/onevdc_helper.rb \
+                   src/cli/one_helper/oneacct_helper.rb \
+                   src/cli/one_helper/onesecgroup_helper.rb"
 
 CLI_BIN_FILES="src/cli/onevm \
                src/cli/onehost \
@@ -1458,7 +1492,10 @@ CLI_BIN_FILES="src/cli/onevm \
                src/cli/onezone \
                src/cli/oneflow \
                src/cli/oneflow-template \
-               src/cli/oneacct"
+               src/cli/oneacct \
+               src/cli/onesecgroup \
+               src/cli/oneshowback \
+               src/cli/onevdc"
 
 CLI_CONF_FILES="src/cli/etc/onegroup.yaml \
                 src/cli/etc/onehost.yaml \
@@ -1471,7 +1508,10 @@ CLI_CONF_FILES="src/cli/etc/onegroup.yaml \
                 src/cli/etc/onedatastore.yaml \
                 src/cli/etc/onecluster.yaml \
                 src/cli/etc/onezone.yaml \
-                src/cli/etc/oneacct.yaml"
+                src/cli/etc/oneacct.yaml \
+                src/cli/etc/onesecgroup.yaml \
+                src/cli/etc/oneshowback.yaml \
+                src/cli/etc/onevdc.yaml"
 
 #-----------------------------------------------------------------------------
 # Sunstone files
@@ -1489,8 +1529,10 @@ SUNSTONE_ETC_FILES="src/sunstone/etc/sunstone-server.conf \
 SUNSTONE_ETC_VIEW_FILES="src/sunstone/etc/sunstone-views/admin.yaml \
                     src/sunstone/etc/sunstone-views/user.yaml \
                     src/sunstone/etc/sunstone-views/cloud.yaml \
-                    src/sunstone/etc/sunstone-views/vdcadmin.yaml \
-		    src/sunstone/etc/sunstone-views/vcenter.yaml"
+                    src/sunstone/etc/sunstone-views/cloud_vcenter.yaml \
+                    src/sunstone/etc/sunstone-views/groupadmin.yaml \
+                    src/sunstone/etc/sunstone-views/groupadmin_vcenter.yaml \
+                    src/sunstone/etc/sunstone-views/admin_vcenter.yaml"
 
 SUNSTONE_MODELS_FILES="src/sunstone/models/OpenNebulaJSON.rb \
                        src/sunstone/models/SunstoneServer.rb \
@@ -1509,11 +1551,14 @@ SUNSTONE_MODELS_JSON_FILES="src/sunstone/models/OpenNebulaJSON/HostJSON.rb \
                     src/sunstone/models/OpenNebulaJSON/ClusterJSON.rb \
                     src/sunstone/models/OpenNebulaJSON/DatastoreJSON.rb \
                     src/sunstone/models/OpenNebulaJSON/VirtualNetworkJSON.rb \
-                    src/sunstone/models/OpenNebulaJSON/ZoneJSON.rb"
+                    src/sunstone/models/OpenNebulaJSON/ZoneJSON.rb \
+                    src/sunstone/models/OpenNebulaJSON/SecurityGroupJSON.rb \
+                    src/sunstone/models/OpenNebulaJSON/VdcJSON.rb"
 
 SUNSTONE_VIEWS_FILES="src/sunstone/views/index.erb \
                       src/sunstone/views/login.erb \
                       src/sunstone/views/vnc.erb \
+                      src/sunstone/views/spice.erb \
                       src/sunstone/views/_login_standard.erb \
                       src/sunstone/views/_login_x509.erb"
 
@@ -1545,7 +1590,9 @@ SUNSTONE_PUBLIC_JS_PLUGINS_FILES="\
                         src/sunstone/public/js/plugins/oneflow-services.js \
                         src/sunstone/public/js/plugins/oneflow-templates.js \
                         src/sunstone/public/js/plugins/support-tab.js \
-                        src/sunstone/public/js/plugins/zones-tab.js"
+                        src/sunstone/public/js/plugins/zones-tab.js \
+                        src/sunstone/public/js/plugins/secgroups-tab.js \
+                        src/sunstone/public/js/plugins/vdcs-tab.js"
 
 SUNSTONE_ROUTES_FILES="src/sunstone/routes/oneflow.rb \
   src/sunstone/routes/vcenter.rb \
@@ -1556,6 +1603,7 @@ SUNSTONE_ROUTES_FILES="src/sunstone/routes/oneflow.rb \
 
 SUNSTONE_PUBLIC_NEW_VENDOR_JQUERY="\
     src/sunstone/public/bower_components/jquery/dist/jquery.min.js \
+    src/sunstone/public/bower_components/jquery/dist/jquery.min.map \
     src/sunstone/public/bower_components/jquery-migrate/jquery-migrate.min.js"
 
 SUNSTONE_PUBLIC_NEW_VENDOR_DATATABLES="\
@@ -1570,6 +1618,7 @@ SUNSTONE_PUBLIC_NEW_VENDOR_FOUNDATION="\
 
 SUNSTONE_PUBLIC_NEW_VENDOR_JGROWL="\
     src/sunstone/public/bower_components/jgrowl/jquery.jgrowl.min.js \
+    src/sunstone/public/bower_components/jgrowl/jquery.jgrowl.map \
     src/sunstone/public/bower_components/jgrowl/jquery.jgrowl.min.css"
 
 
@@ -1593,12 +1642,13 @@ SUNSTONE_PUBLIC_NEW_VENDOR_FONTAWESOME_CSS="\
 SUNSTONE_PUBLIC_NEW_VENDOR_FONTAWESOME_FONT="\
     src/sunstone/public/bower_components/fontawesome/fonts/fontawesome-webfont.eot \
     src/sunstone/public/bower_components/fontawesome/fonts/fontawesome-webfont.woff \
+    src/sunstone/public/bower_components/fontawesome/fonts/fontawesome-webfont.woff2 \
     src/sunstone/public/bower_components/fontawesome/fonts/fontawesome-webfont.ttf \
     src/sunstone/public/bower_components/fontawesome/fonts/fontawesome-webfont.svg \
     src/sunstone/public/bower_components/fontawesome/fonts/FontAwesome.otf"
 
 SUNSTONE_PUBLIC_VENDOR_NOVNC="\
-    src/sunstone/public/bower_components/no-vnc/include/base.css \
+    src/sunstone/public/bower_components/no-vnc/include/novnc-custom.css \
     src/sunstone/public/bower_components/no-vnc/include/base64.js \
     src/sunstone/public/bower_components/no-vnc/include/black.css \
     src/sunstone/public/bower_components/no-vnc/include/blue.css \
@@ -1608,6 +1658,7 @@ SUNSTONE_PUBLIC_VENDOR_NOVNC="\
     src/sunstone/public/bower_components/no-vnc/include/jsunzip.js \
     src/sunstone/public/bower_components/no-vnc/include/keyboard.js \
     src/sunstone/public/bower_components/no-vnc/include/keysymdef.js \
+    src/sunstone/public/bower_components/no-vnc/include/keysym.js \
     src/sunstone/public/bower_components/no-vnc/include/logo.js \
     src/sunstone/public/bower_components/no-vnc/include/Orbitron700.ttf \
     src/sunstone/public/bower_components/no-vnc/include/Orbitron700.woff \
@@ -1623,6 +1674,37 @@ SUNSTONE_PUBLIC_VENDOR_NOVNC_WEBSOCKET="\
     src/sunstone/public/bower_components/no-vnc/include/web-socket-js/swfobject.js \
     src/sunstone/public/bower_components/no-vnc/include/web-socket-js/WebSocketMain.swf"
 
+SUNSTONE_PUBLIC_VENDOR_SPICE="\
+    src/sunstone/public/bower_components/spice-html5/spicearraybuffer.js \
+    src/sunstone/public/bower_components/spice-html5/enums.js \
+    src/sunstone/public/bower_components/spice-html5/atKeynames.js \
+    src/sunstone/public/bower_components/spice-html5/utils.js \
+    src/sunstone/public/bower_components/spice-html5/png.js \
+    src/sunstone/public/bower_components/spice-html5/lz.js \
+    src/sunstone/public/bower_components/spice-html5/quic.js \
+    src/sunstone/public/bower_components/spice-html5/bitmap.js \
+    src/sunstone/public/bower_components/spice-html5/spicedataview.js \
+    src/sunstone/public/bower_components/spice-html5/spicetype.js \
+    src/sunstone/public/bower_components/spice-html5/spicemsg.js \
+    src/sunstone/public/bower_components/spice-html5/wire.js \
+    src/sunstone/public/bower_components/spice-html5/spiceconn.js \
+    src/sunstone/public/bower_components/spice-html5/display.js \
+    src/sunstone/public/bower_components/spice-html5/main.js \
+    src/sunstone/public/bower_components/spice-html5/inputs.js \
+    src/sunstone/public/bower_components/spice-html5/webm.js \
+    src/sunstone/public/bower_components/spice-html5/playback.js \
+    src/sunstone/public/bower_components/spice-html5/simulatecursor.js \
+    src/sunstone/public/bower_components/spice-html5/cursor.js \
+    src/sunstone/public/bower_components/spice-html5/thirdparty/jsbn.js \
+    src/sunstone/public/bower_components/spice-html5/thirdparty/rsa.js \
+    src/sunstone/public/bower_components/spice-html5/thirdparty/prng4.js \
+    src/sunstone/public/bower_components/spice-html5/thirdparty/rng.js \
+    src/sunstone/public/bower_components/spice-html5/thirdparty/sha1.js \
+    src/sunstone/public/bower_components/spice-html5/ticket.js \
+    src/sunstone/public/bower_components/spice-html5/resize.js \
+    src/sunstone/public/bower_components/spice-html5/filexfer.js \
+    src/sunstone/public/bower_components/spice-html5/spice-custom.css"
+
 # end bower
 
 
@@ -1802,6 +1884,7 @@ ONEFLOW_LIB_MODELS_FILES="src/flow/lib/models/role.rb \
 #-----------------------------------------------------------------------------
 
 MAN_FILES="share/man/oneacct.1.gz \
+        share/man/oneshowback.1.gz \
         share/man/oneacl.1.gz \
         share/man/onehost.1.gz \
         share/man/oneimage.1.gz \
@@ -1814,8 +1897,11 @@ MAN_FILES="share/man/oneacct.1.gz \
         share/man/onedatastore.1.gz \
         share/man/onecluster.1.gz \
         share/man/onezone.1.gz \
+        share/man/onevcenter.1.gz \
         share/man/oneflow.1.gz \
         share/man/oneflow-template.1.gz \
+        share/man/onesecgroup.1.gz \
+        share/man/onevdc.1.gz \
         share/man/econe-allocate-address.1.gz \
         share/man/econe-associate-address.1.gz \
         share/man/econe-attach-volume.1.gz \
@@ -1839,6 +1925,12 @@ MAN_FILES="share/man/oneacct.1.gz \
         share/man/econe-terminate-instances.1.gz \
         share/man/econe-upload.1.gz"
 
+#-----------------------------------------------------------------------------
+# Docs Files
+#-----------------------------------------------------------------------------
+
+DOCS_FILES="LICENSE NOTICE README.md"
+
 #-----------------------------------------------------------------------------
 # Ruby VENDOR files
 #-----------------------------------------------------------------------------
