@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -83,18 +83,17 @@ AclManager::AclManager(
         string error_str;
 
         // Users in group USERS can create standard resources
-        // @1 VM+NET+IMAGE+TEMPLATE+DOCUMENT/* CREATE #<local-zone>
+        // @1 VM+IMAGE+TEMPLATE+DOCUMENT/* CREATE *
         add_rule(AclRule::GROUP_ID |
                     1,
                  AclRule::ALL_ID |
                     PoolObjectSQL::VM |
-                    PoolObjectSQL::NET |
                     PoolObjectSQL::IMAGE |
                     PoolObjectSQL::TEMPLATE |
-                    PoolObjectSQL::DOCUMENT,
+                    PoolObjectSQL::DOCUMENT |
+                    PoolObjectSQL::SECGROUP,
                  AuthRequest::CREATE,
-                 AclRule::INDIVIDUAL_ID |
-                     zone_id,
+                 AclRule::ALL_ID,
                  error_str);
 
         // * ZONE/* USE *
