@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -640,6 +640,7 @@ void VirtualMachineDeploy::request_execute(xmlrpc_c::paramList const& paramList,
     int  id      = xmlrpc_c::value_int(paramList.getInt(1));
     int  hid     = xmlrpc_c::value_int(paramList.getInt(2));
     bool enforce = false;
+    bool imported= false;
     int  ds_id   = -1;
 
     if ( paramList.size() > 3 )
@@ -688,6 +689,10 @@ void VirtualMachineDeploy::request_execute(xmlrpc_c::paramList const& paramList,
     {
         ds_id = vm->get_ds_id();
     }
+    else if (!vm->get_deploy_id().empty()) //deploy_id && not a Stopped VM
+    {
+        imported = true;
+    }
 
     vm->unlock();
 
@@ -795,7 +800,14 @@ void VirtualMachineDeploy::request_execute(xmlrpc_c::paramList const& paramList,
         return;
     }
 
-    dm->deploy(vm);
+    if (!imported)
+    {
+        dm->deploy(vm);    
+    }
+    else
+    {
+        dm->import(vm);
+    }
 
     vm->unlock();
 
@@ -2219,3 +2231,41 @@ void VirtualMachineRecover::request_execute(
 
 /* -------------------------------------------------------------------------- */
 /* -------------------------------------------------------------------------- */
+
+void VirtualMachinePoolCalculateShowback::request_execute(
+        xmlrpc_c::paramList const& paramList,
+        RequestAttributes& att)
+{
+    int start_month = xmlrpc_c::value_int(paramList.getInt(1));
+    int start_year  = xmlrpc_c::value_int(paramList.getInt(2));
+    int end_month   = xmlrpc_c::value_int(paramList.getInt(3));
+    int end_year    = xmlrpc_c::value_int(paramList.getInt(4));
+
+    ostringstream oss;
+    string        where;
+    int           rc;
+    string        error_str;
+
+    if ( att.gid != 0 )
+    {
+        failure_response(AUTHORIZATION,
+                         authorization_error("Action reserved for group 0 only", att),
+                         att);
+        return;
+    }
+
+    rc = (static_cast<VirtualMachinePool *>(pool))->calculate_showback(
+                    start_month, start_year, end_month, end_year, error_str);
+
+    if (rc != 0)
+    {
+        failure_response(AUTHORIZATION,
+                         request_error(error_str, ""),
+                         att);
+        return;
+    }
+
+    success_response("", att);
+
+    return;
+}
