@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -315,7 +315,7 @@ void VirtualMachineXML::init_storage_usage()
 
     system_ds_usage = 0;
 
-    vm_template->remove("DISK", disks);
+    int num = vm_template->remove("DISK", disks);
 
     for (it=disks.begin(); it != disks.end(); it++)
     {
@@ -373,6 +373,11 @@ void VirtualMachineXML::init_storage_usage()
             } // else st == NONE
         }
     }
+
+    for (int i = 0; i < num ; i++)
+    {
+        delete disks[i];
+    }
 }
 
 /* -------------------------------------------------------------------------- */
