@@ -1,5 +1,5 @@
 /* -------------------------------------------------------------------------- */
-/* Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        */
+/* Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        */
 /*                                                                            */
 /* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
 /* not use this file except in compliance with the License. You may obtain    */
@@ -164,6 +164,11 @@ var OpenNebula = {
                                "DELETE",
                                "USED_PERS"][value]);
                     break;
+                case "DATASTORE":
+                case "datastore":
+                    state = tr(["ON",
+                               "OFF"][value]);
+                    break;
                 case "VM_MIGRATE_REASON":
                 case "vm_migrate_reason":
                     state = tr(["NONE",
@@ -344,6 +349,7 @@ var OpenNebula = {
                 type: "POST",
                 dataType: "json",
                 data: JSON.stringify(data),
+                contentType: "application/json; charset=utf-8",
                 success: function(response){
                     OpenNebula.Helper.clear_cache(cache_name);
 
@@ -576,6 +582,7 @@ var OpenNebula = {
             $.ajax({
                 url: req_path + "/" + id + "/action",
                 type: "POST",
+                contentType: "application/json; charset=utf-8",
                 data: JSON.stringify(action),
                 success: function(){
                     OpenNebula.Helper.clear_cache(cache_name);
@@ -639,6 +646,31 @@ var OpenNebula = {
                 }
             });
         },
+
+        "showback": function(params, resource, path){
+            var callback = params.success;
+            var callback_error = params.error;
+            var data = params.data;
+
+            var method = "showback";
+            var request = OpenNebula.Helper.request(resource,method, data);
+
+            var url = path ? path : resource.toLowerCase() + "/showback";
+
+            $.ajax({
+                url: url,
+                type: "GET",
+                data: data,
+                dataType: "json",
+                success: function(response){
+                    return callback ? callback(request, response) : null;
+                },
+                error: function(response){
+                    return callback_error ?
+                        callback_error(request, OpenNebula.Error(response)) : null;
+                }
+            });
+        }
     },
 
     "Auth": {
@@ -717,6 +749,9 @@ var OpenNebula = {
         "list": function(params){
             OpenNebula.Action.list(params,OpenNebula.Host.resource);
         },
+        "list_in_zone" : function(params){
+            OpenNebula.Action.list_in_zone(params,OpenNebula.Host.resource);
+        },
         "show": function(params){
             OpenNebula.Action.show(params,OpenNebula.Host.resource);
         },
@@ -763,6 +798,9 @@ var OpenNebula = {
         "list": function(params){
             OpenNebula.Action.list(params,OpenNebula.Network.resource);
         },
+        "list_in_zone" : function(params){
+            OpenNebula.Action.list_in_zone(params,OpenNebula.Network.resource);
+        },
         "show": function(params){
             OpenNebula.Action.show(params,OpenNebula.Network.resource);
         },
@@ -1045,6 +1083,9 @@ var OpenNebula = {
         "accounting": function(params){
             OpenNebula.Action.accounting(params,OpenNebula.VM.resource);
         },
+        "showback": function(params){
+            OpenNebula.Action.showback(params,OpenNebula.VM.resource);
+        }
     },
 
     "Group": {
@@ -1106,13 +1147,13 @@ var OpenNebula = {
         "accounting" : function(params){
             OpenNebula.Action.monitor(params,OpenNebula.Group.resource,false);
         },
-        "add_provider" : function(params){
+        "add_admin" : function(params){
             var action_obj = params.data.extra_param;
-            OpenNebula.Action.simple_action(params,OpenNebula.Group.resource,"add_provider",action_obj);
+            OpenNebula.Action.simple_action(params,OpenNebula.Group.resource,"add_admin",action_obj);
         },
-        "del_provider" : function(params){
+        "del_admin" : function(params){
             var action_obj = params.data.extra_param;
-            OpenNebula.Action.simple_action(params,OpenNebula.Group.resource,"del_provider",action_obj);
+            OpenNebula.Action.simple_action(params,OpenNebula.Group.resource,"del_admin",action_obj);
         }
     },
 
@@ -1429,6 +1470,9 @@ var OpenNebula = {
         "list" : function(params){
             OpenNebula.Action.list(params,OpenNebula.Datastore.resource);
         },
+        "list_in_zone" : function(params){
+            OpenNebula.Action.list_in_zone(params,OpenNebula.Datastore.resource);
+        },
         "show" : function(params){
             OpenNebula.Action.show(params,OpenNebula.Datastore.resource);
         },
@@ -1461,6 +1505,12 @@ var OpenNebula = {
                                             OpenNebula.Datastore.resource,
                                             "rename",
                                             action_obj);
+        },
+        "enable": function(params){
+            OpenNebula.Action.simple_action(params,OpenNebula.Datastore.resource,"enable");
+        },
+        "disable": function(params){
+            OpenNebula.Action.simple_action(params,OpenNebula.Datastore.resource,"disable");
         }
     },
 
@@ -1498,6 +1548,138 @@ var OpenNebula = {
         }
     },
 
+    "SecurityGroup" : {
+        "resource" : "SECURITY_GROUP",
+
+        "create" : function(params){
+            OpenNebula.Action.create(params,OpenNebula.SecurityGroup.resource);
+        },
+        "del" : function(params){
+            OpenNebula.Action.del(params,OpenNebula.SecurityGroup.resource);
+        },
+        "list" : function(params){
+            OpenNebula.Action.list(params,OpenNebula.SecurityGroup.resource);
+        },
+        "show" : function(params){
+            OpenNebula.Action.show(params,OpenNebula.SecurityGroup.resource);
+        },
+        "chown" : function(params){
+            OpenNebula.Action.chown(params,OpenNebula.SecurityGroup.resource);
+        },
+        "chgrp" : function(params){
+            OpenNebula.Action.chgrp(params,OpenNebula.SecurityGroup.resource);
+        },
+        "chmod" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,
+                                            OpenNebula.SecurityGroup.resource,
+                                            "chmod",
+                                            action_obj);
+        },
+        "update": function(params){
+            var action_obj = {"template_raw" : params.data.extra_param };
+            OpenNebula.Action.simple_action(params,
+                                            OpenNebula.SecurityGroup.resource,
+                                            "update",
+                                            action_obj);
+        },
+        "fetch_template" : function(params){
+            OpenNebula.Action.show(params,OpenNebula.SecurityGroup.resource,"template");
+        },
+        "clone" : function(params) {
+            var name = params.data.extra_param ? params.data.extra_param : "";
+            var action_obj = { "name" : name };
+            OpenNebula.Action.simple_action(params,OpenNebula.SecurityGroup.resource, "clone", action_obj);
+        },
+        "rename" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,
+                                            OpenNebula.SecurityGroup.resource,
+                                            "rename",
+                                            action_obj);
+        }
+    },
+
+
+    "Vdc" : {
+        "resource" : "VDC",
+
+        "create" : function(params){
+            OpenNebula.Action.create(params,OpenNebula.Vdc.resource);
+        },
+        "del" : function(params){
+            OpenNebula.Action.del(params,OpenNebula.Vdc.resource);
+        },
+        "list" : function(params){
+            OpenNebula.Action.list(params,OpenNebula.Vdc.resource);
+        },
+        "show" : function(params){
+            OpenNebula.Action.show(params,OpenNebula.Vdc.resource);
+        },
+        "update" : function(params){
+            var action_obj = {"template_raw" : params.data.extra_param };
+            OpenNebula.Action.simple_action(params,
+                                     OpenNebula.Vdc.resource,
+                                     "update",
+                                     action_obj);
+        },
+        "fetch_template" : function(params){
+            OpenNebula.Action.show(params,OpenNebula.Vdc.resource,"template");
+        },
+        "rename" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,
+                                            OpenNebula.Vdc.resource,
+                                            "rename",
+                                            action_obj);
+        },
+
+        "add_group" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"add_group",action_obj);
+        },
+        "del_group" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"del_group",action_obj);
+        },
+
+        "add_cluster" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"add_cluster",action_obj);
+        },
+        "del_cluster" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"del_cluster",action_obj);
+        },
+
+        "add_host" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"add_host",action_obj);
+        },
+        "del_host" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"del_host",action_obj);
+        },
+
+        "add_datastore" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"add_datastore",action_obj);
+        },
+        "del_datastore" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"del_datastore",action_obj);
+        },
+
+        "add_vnet" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"add_vnet",action_obj);
+        },
+        "del_vnet" : function(params){
+            var action_obj = params.data.extra_param;
+            OpenNebula.Action.simple_action(params,OpenNebula.Vdc.resource,"del_vnet",action_obj);
+        }
+    },
+
     "Marketplace" : {
         "resource" : "MARKETPLACE",
 
@@ -1785,12 +1967,13 @@ var OpenNebula = {
                 url: OpenNebula.Role.path + "/" + params.data.id,
                 type: "PUT",
                 dataType: "json",
+                contentType: "application/json; charset=utf-8",
                 data: JSON.stringify(params.data.extra_param),
                 success: function(response){
                     return params.success ? params.success(request, response) : null;
                 },
                 error: function(response){
-                    return params.error ? params.error(request, OpenNebula.Error(res)) : null;
+                    return params.error ? params.error(request, OpenNebula.Error(response)) : null;
                 }
             });
         }
