@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # -------------------------------------------------------------------------- #
-# Copyright 2002-2014, OpenNebula Project (OpenNebula.org), C12G Labs        #
+# Copyright 2002-2015, OpenNebula Project (OpenNebula.org), C12G Labs        #
 #                                                                            #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may    #
 # not use this file except in compliance with the License. You may obtain    #
@@ -218,14 +218,10 @@ helpers do
                 session[:lang] = $conf[:lang]
             end
 
-            if user['TEMPLATE/VNC_WSS']
-                session[:vnc_wss] = user['TEMPLATE/VNC_WSS']
-            else
-                wss = $conf[:vnc_proxy_support_wss]
-                #limit to yes,no options
-                session[:vnc_wss] = (wss == true || wss == "yes" || wss == "only" ?
-                                 "yes" : "no")
-            end
+            wss = $conf[:vnc_proxy_support_wss]
+            #limit to yes,no options
+            session[:vnc_wss] = (wss == true || wss == "yes" || wss == "only" ?
+                             "yes" : "no")
 
             if user['TEMPLATE/TABLE_ORDER']
                 session[:table_order] = user['TEMPLATE/TABLE_ORDER']
@@ -235,6 +231,8 @@ helpers do
 
             if user['TEMPLATE/DEFAULT_VIEW']
                 session[:default_view] = user['TEMPLATE/DEFAULT_VIEW']
+            elsif group.contains_admin(user.id) && group['TEMPLATE/GROUP_ADMIN_DEFAULT_VIEW']
+                session[:default_view] = group['TEMPLATE/GROUP_ADMIN_DEFAULT_VIEW']
             elsif group['TEMPLATE/DEFAULT_VIEW']
                 session[:default_view] = group['TEMPLATE/DEFAULT_VIEW']
             else
@@ -277,7 +275,7 @@ before do
     @request_body = request.body.read
     request.body.rewind
 
-    unless %w(/ /login /vnc).include?(request.path)
+    unless %w(/ /login /vnc /spice).include?(request.path)
         halt 401 unless authorized? && valid_csrftoken?
     end
 
@@ -354,6 +352,15 @@ get '/vnc' do
     end
 end
 
+get '/spice' do
+    content_type 'text/html', :charset => 'utf-8'
+    if !authorized?
+        erb :login
+    else
+        erb :spice
+    end
+end
+
 ##############################################################################
 # Login
 ##############################################################################
@@ -446,6 +453,13 @@ get '/vm/accounting' do
     @SunstoneServer.get_vm_accounting(params)
 end
 
+##############################################################################
+# Showback
+##############################################################################
+
+get '/vm/showback' do
+    @SunstoneServer.get_vm_showback(params)
+end
 
 ##############################################################################
 # Marketplace
