@@ -1,5 +1,5 @@
 # ---------------------------------------------------------------------------- #
-# Copyright 2010-2014, C12G Labs S.L                                           #
+# Copyright 2010-2015, C12G Labs S.L                                           #
 #                                                                              #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may      #
 # not use this file except in compliance with the License. You may obtain      #
@@ -129,7 +129,7 @@ class VIClient
 
     ########################################################################
     # Initialize a VIConnection based just on the VIM parameters. The
-    # OpenNebula client is also initilialize
+    # OpenNebula client is also initilialized
     ########################################################################
     def self.new_connection(user_opts)
 
@@ -166,6 +166,23 @@ class VIClient
         end
     end
 
+    ########################################################################
+    # Searches the associated vmFolder of the DataCenter for the current
+    # connection. Returns a RbVmomi::VIM::VirtualMachine or nil if not found
+    # @param vm_name [String] the UUID of the VM or VM Template
+    ########################################################################
+    def find_vm(vm_name)
+        vms = get_entities(@dc.vmFolder, 'VirtualMachine')
+
+        return vms.find do |v|
+            begin
+                v.name == vm_name
+            rescue ManagedObjectNotFound
+                false
+            end
+        end
+    end    
+
     ########################################################################
     # Builds a hash with the DataCenter / ClusterComputeResource hierarchy
     # for this VCenter.
@@ -187,12 +204,18 @@ class VIClient
 
     ########################################################################
     # Builds a hash with the Datacenter / VM Templates for this VCenter
+    # @param one_client [OpenNebula::Client] Use this client instead of @one
     # @return [Hash] in the form
     #   { dc_name [String] => }
     ########################################################################
-    def vm_templates
+    def vm_templates(one_client=nil)
         vm_templates = {}
 
+        tpool = OpenNebula::TemplatePool.new(
+            (one_client||@one), OpenNebula::Pool::INFO_ALL)
+        rc = tpool.info
+        # TODO check error
+
         datacenters = get_entities(@root, 'Datacenter')
 
         datacenters.each { |dc|
@@ -205,12 +228,16 @@ class VIClient
             tmp.each { |t|
                 vi_tmp = VCenterVm.new(self, t)
 
-                one_tmp << {
-                    :name => vi_tmp.vm.name,
-                    :uuid => vi_tmp.vm.config.uuid,
-                    :host => vi_tmp.vm.runtime.host.parent.name,
-                    :one  => vi_tmp.to_one
-                }
+                if !tpool["VMTEMPLATE/TEMPLATE/PUBLIC_CLOUD[\
+                        TYPE=\"vcenter\" \
+                        and VM_TEMPLATE=\"#{vi_tmp.vm.config.uuid}\"]"]
+                    one_tmp << {
+                        :name => vi_tmp.vm.name,
+                        :uuid => vi_tmp.vm.config.uuid,
+                        :host => vi_tmp.vm.runtime.host.parent.name,
+                        :one  => vi_tmp.to_one
+                    }
+                end
             }
 
             vm_templates[dc.name] = one_tmp
@@ -219,6 +246,184 @@ class VIClient
         return vm_templates
     end
 
+    ########################################################################
+    # Builds a hash with the Datacenter / Virtual Machines for this VCenter
+    # @param one_client [OpenNebula::Client] Use this client instead of @one
+    # @return [Hash] in the form
+    #   { dc_name [String] => }
+    ########################################################################
+    def running_vms(one_client=nil)
+        running_vms = {}
+
+        vmpool = OpenNebula::VirtualMachinePool.new(
+            (one_client||@one), OpenNebula::Pool::INFO_ALL)
+        rc = vmpool.info 
+
+        hostpool = OpenNebula::HostPool.new((one_client||@one))
+        rc = hostpool.info
+        # TODO check error
+
+        datacenters = get_entities(@root, 'Datacenter')
+
+        datacenters.each { |dc|
+            vms     = get_entities(dc.vmFolder, 'VirtualMachine')
+            ccrs    = get_entities(dc.hostFolder, 'ClusterComputeResource')
+
+            vm_list = vms.select { |v| 
+                # Get rid of VM Templates and VMs not in running state
+                v.config &&
+                v.config.template != true &&  
+                v.summary.runtime.powerState == "poweredOn"
+            }
+
+            one_tmp = []
+
+            vm_list.each { |v|
+                vi_tmp = VCenterVm.new(self, v)
+
+                # Do not reimport VMs deployed by OpenNebula
+                # since the core will get confused with the IDs
+                next if vi_tmp.vm.name.match(/one-\d/) 
+
+                container_hostname = vi_tmp.vm.runtime.host.parent.name
+
+                cluster_name = ccrs.collect { |c|
+                  found_host=c.host.select {|h|
+                           h.parent.name == container_hostname}
+                   found_host.first.parent.name if found_host.size > 0
+                }.first
+
+                if !vmpool["VM/USER_TEMPLATE/PUBLIC_CLOUD[\
+                        TYPE=\"vcenter\" \
+                        and VM_TEMPLATE=\"#{vi_tmp.vm.config.uuid}\"]"]
+
+                    host_id = name_to_id(container_hostname,hostpool,"HOST")[1]
+
+                    one_tmp << {
+                        :name => vi_tmp.vm.name,
+                        :uuid => vi_tmp.vm.config.uuid,
+                        :host => container_hostname,
+                        :host_id => host_id,
+                        :one  => vi_tmp.vm_to_one
+                    }
+                end
+            }
+
+            running_vms[dc.name] = one_tmp
+        }
+
+        return running_vms
+    end
+
+    def name_to_id(name, pool, ename)
+            objects=pool.select {|object| object.name==name }
+
+            if objects.length>0
+                if objects.length>1
+                    return -1, "There are multiple #{ename}s with name #{name}."
+                else
+                    result = objects.first.id
+                end
+            else
+                return -1, "#{ename} named #{name} not found."
+            end
+
+            return 0, result
+    end
+
+    ########################################################################
+    # Builds a hash with the Datacenter / CCR (Distributed)Networks 
+    # for this VCenter
+    # @param one_client [OpenNebula::Client] Use this client instead of @one
+    # @return [Hash] in the form
+    #   { dc_name [String] => Networks [Array] }
+    ########################################################################
+    def vcenter_networks(one_client=nil)
+        vcenter_networks = {}
+
+        vnpool = OpenNebula::VirtualNetworkPool.new(
+            (one_client||@one), OpenNebula::Pool::INFO_ALL)
+        rc = vnpool.info
+        # TODO check error
+        # 
+        datacenters = get_entities(@root, 'Datacenter')
+
+        datacenters.each { |dc|
+            networks = get_entities(dc.networkFolder, 'Network' )
+            one_nets = []
+
+            networks.each { |n|
+                if !vnpool["VNET[BRIDGE=\"#{n[:name]}\"]/\
+                        TEMPLATE[VCENTER_TYPE=\"Port Group\"]"]
+                    one_nets << {
+                        :name   => n.name,
+                        :bridge => n.name,
+                        :type   => "Port Group",
+                        :one    => "NAME   = \"#{n[:name]}\"\n" \
+                                   "BRIDGE = \"#{n[:name]}\"\n" \
+                                   "VCENTER_TYPE = \"Port Group\""
+                    }
+                end
+            }
+
+            networks = get_entities(dc.networkFolder,
+                                    'DistributedVirtualPortgroup' )
+
+            networks.each { |n|
+                if !vnpool["VNET[BRIDGE=\"#{n[:name]}\"]/\
+                        TEMPLATE[VCENTER_TYPE=\"Distributed Port Group\"]"]
+                    vnet_template = "NAME   = \"#{n[:name]}\"\n" \
+                                    "BRIDGE = \"#{n[:name]}\"\n" \
+                                    "VCENTER_TYPE = \"Distributed Port Group\""
+
+
+                    default_pc = n.config.defaultPortConfig
+
+                    has_vlan = false
+                    vlan_str = ""
+
+                    if default_pc.methods.include? :vlan
+                       has_vlan = default_pc.vlan.methods.include? :vlanId
+                    end
+
+                    if has_vlan
+                        vlan     = n.config.defaultPortConfig.vlan.vlanId
+
+                        if vlan != 0
+                            if vlan.is_a? Array
+                                vlan.each{|v|
+                                    vlan_str += v.start.to_s + ".." +
+                                                v.end.to_s + ","
+                                }
+                                vlan_str.chop!
+                            else
+                                vlan_str = vlan.to_s
+                            end
+                        end
+                    end
+
+                    if !vlan_str.empty?
+                        vnet_template << "VLAN=\"YES\"\n" \
+                                         "VLAN_ID=#{vlan_str}\n"
+                    end
+
+                    one_net = {:name   => n.name,
+                               :bridge => n.name,
+                               :type   => "Distributed Port Group",
+                               :one    => vnet_template}
+
+                    one_net[:vlan] = vlan_str if !vlan_str.empty?
+
+                    one_nets << one_net
+                end
+            }
+
+            vcenter_networks[dc.name] = one_nets
+        }
+
+        return vcenter_networks
+    end
+
     def self.translate_hostname(hostname)
         host_pool = OpenNebula::HostPool.new(::OpenNebula::Client.new())
         rc        = host_pool.info
@@ -433,9 +638,12 @@ class VCenterHost < ::OpenNebula::Host
             vm = VCenterVm.new(@client, v)
             vm.monitor
 
+            next if !vm.vm.config
+
             str_info << "\nVM = ["
             str_info << "ID=#{number},"
-            str_info << "DEPLOY_ID=\"#{name}\","
+            str_info << "DEPLOY_ID=\"#{vm.vm.config.uuid}\","
+            str_info << "VM_NAME=\"#{name}\","
             str_info << "POLL=\"#{vm.info}\"]"
         }
 
@@ -552,7 +760,7 @@ class VCenterVm
 
     ############################################################################
     # Resets a VM
-    #  @param deploy_id vcenter identifier of the VM
+    #  @param deploy_id vcetranslate_hostnamnter identifier of the VM
     #  @param hostname name of the host (equals the vCenter cluster)
     ############################################################################
     def self.reset(deploy_id, hostname)
@@ -616,6 +824,24 @@ class VCenterVm
         return snapshot_name
     end
 
+    ############################################################################
+    # Find VM snapshot
+    #  @param list root list of VM snapshots
+    #  @param snaphot_name name of the snapshot
+    ############################################################################
+    def self.find_snapshot(list, snapshot_name)
+        list.each do |i|
+            if i.name == snapshot_name
+                return i.snapshot
+            elsif !i.childSnapshotList.empty?
+                snap = find_snapshot(i.childSnapshotList, snapshot_name)
+                return snap if snap
+            end
+        end
+
+        nil
+    end
+
     ############################################################################
     # Delete VM snapshot
     #  @param deploy_id vcenter identifier of the VM
@@ -628,13 +854,14 @@ class VCenterVm
 
         vm          = connection.find_vm_template(deploy_id)
 
-        snapshot = vm.snapshot.rootSnapshotList.find {|s|
-                      s.name == snapshot_name
-                   }.snapshot
+        list = vm.snapshot.rootSnapshotList
+
+        snapshot = find_snapshot(list, snapshot_name)
+        return nil if !snapshot
 
         delete_snapshot_hash = {
             :_this => snapshot,
-            :removeChildren => true
+            :removeChildren => false
         }
 
         snapshot.RemoveSnapshot_Task(delete_snapshot_hash).wait_for_completion
@@ -652,9 +879,10 @@ class VCenterVm
 
         vm          = connection.find_vm_template(deploy_id)
 
-        snapshot = vm.snapshot.rootSnapshotList.find {|s|
-                      s.name == snapshot_name
-                   }.snapshot
+        list = vm.snapshot.rootSnapshotList
+
+        snapshot = find_snapshot(list, snapshot_name)
+        return nil if !snapshot
 
         revert_snapshot_hash = {
             :_this => snapshot
@@ -663,6 +891,53 @@ class VCenterVm
         snapshot.RevertToSnapshot_Task(revert_snapshot_hash).wait_for_completion
     end
 
+    ############################################################################
+    # Attach NIC to a VM
+    #  @param deploy_id vcenter identifier of the VM
+    #  @param mac MAC address of the NIC to be attached
+    #  @param bridge name of the Network in vCenter
+    #  @param model model of the NIC to be attached
+    #  @param host hostname of the ESX where the VM is running
+    ############################################################################
+    def self.attach_nic(deploy_id, mac, bridge, model, host)
+        hid         = VIClient::translate_hostname(host)
+        connection  = VIClient.new(hid)
+
+        vm          = connection.find_vm_template(deploy_id)
+
+        spec_hash   = calculate_addnic_spec(vm, mac, bridge, model)
+
+        spec        = RbVmomi::VIM.VirtualMachineConfigSpec({:deviceChange => 
+                                                              [spec_hash]})
+
+        vm.ReconfigVM_Task(:spec => spec).wait_for_completion
+    end
+
+    ############################################################################
+    # Detach NIC from a VM
+    ############################################################################
+    def self.detach_nic(deploy_id, mac, host)
+        hid         = VIClient::translate_hostname(host)
+        connection  = VIClient.new(hid)
+
+        vm   = connection.find_vm_template(deploy_id)
+
+        nic  = vm.config.hardware.device.find { |d|
+                is_nic?(d) && (d.macAddress ==  mac)
+        }
+
+        raise "Could not find NIC with mac address #{mac}" if nic.nil?
+
+        spec = {
+            :deviceChange => [
+                :operation => :remove,
+                :device => nic
+            ]
+        }
+
+        vm.ReconfigVM_Task(:spec => spec).wait_for_completion
+    end    
+
     ########################################################################
     #  Initialize the vm monitor information
     ########################################################################
@@ -744,6 +1019,62 @@ class VCenterVm
               "  LISTEN   =\"0.0.0.0\"\n"\
               "]\n"\
          "SCHED_REQUIREMENTS=\"NAME=\\\"#{@vm.runtime.host.parent.name}\\\"\"\n"
+
+        if @vm.config.annotation.nil? || @vm.config.annotation.empty?
+            str << "DESCRIPTION = \"vCenter Template imported by OpenNebula"\
+                " from Cluster #{@vm.runtime.host.parent.name}\"\n"
+        else
+            notes = @vm.config.annotation.gsub("\\", "\\\\").gsub("\"", "\\\"")
+            str << "DESCRIPTION = \"#{notes}\"\n"
+        end
+        
+        case @vm.guest.guestFullName
+            when /CentOS/i
+                str << "LOGO=images/logos/centos.png"
+            when /Debian/i
+                str << "LOGO=images/logos/debian.png"
+            when /Red Hat/i
+                str << "LOGO=images/logos/redhat.png"
+            when /Ubuntu/i
+                str << "LOGO=images/logos/ubuntu.png"
+            when /Windows XP/i
+                str << "LOGO=images/logos/windowsxp.png"
+            when /Windows/i
+                str << "LOGO=images/logos/windows8.png"
+            when /Linux/i
+                str << "LOGO=images/logos/linux.png"
+        end
+
+        return str
+    end
+
+    ########################################################################
+    # Generates an OpenNebula VirtualMachine for this VCenterVm
+    #
+    #
+    ########################################################################
+    def vm_to_one
+        str = "NAME   = \"#{@vm.name}\"\n"\
+              "CPU    = \"#{@vm.config.hardware.numCPU}\"\n"\
+              "vCPU   = \"#{@vm.config.hardware.numCPU}\"\n"\
+              "MEMORY = \"#{@vm.config.hardware.memoryMB}\"\n"\
+              "HYPERVISOR = \"vcenter\"\n"\
+              "PUBLIC_CLOUD = [\n"\
+              "  TYPE        =\"vcenter\",\n"\
+              "  VM_TEMPLATE =\"#{@vm.config.uuid}\"\n"\
+              "]\n"\
+              "IMPORT_VM_ID    = \"#{@vm.config.uuid}\"\n"\
+              "SCHED_REQUIREMENTS=\"NAME=\\\"#{@vm.runtime.host.parent.name}\\\"\"\n"
+
+        if @vm.config.annotation.nil? || @vm.config.annotation.empty?
+            str << "DESCRIPTION = \"vCenter Virtual Machine imported by OpenNebula"\
+                " from Cluster #{@vm.runtime.host.parent.name}\"\n"
+        else
+            notes = @vm.config.annotation.gsub("\\", "\\\\").gsub("\"", "\\\"")
+            str << "DESCRIPTION = \"#{notes}\"\n"
+        end
+
+        return str
     end
 
 private
@@ -768,6 +1099,80 @@ private
         end
     end
 
+    ########################################################################
+    #  Checks if a RbVmomi::VIM::VirtualDevice is a network interface
+    ########################################################################
+    def self.is_nic?(device)
+        !device.class.ancestors.index(RbVmomi::VIM::VirtualEthernetCard).nil?
+    end
+
+    ########################################################################
+    # Returns the spec to reconfig a VM and add a NIC
+    ########################################################################
+    def self.calculate_addnic_spec(vm, mac, bridge, model)
+        model       = model.nil? ? nil : model.downcase
+        network     = vm.runtime.host.network.select{|n| n.name==bridge}
+        backing     = nil
+
+        if network.empty?
+            raise "Network #{bridge} not found in host #{vm.runtime.host.name}"
+        else
+            network = network[0]
+        end
+
+        card_num = 1 # start in one, we want the next avaliable id
+
+        vm.config.hardware.device.each{ |dv|
+            card_num = card_num + 1 if is_nic?(dv)
+        } 
+
+        nic_card = case model
+                        when "virtuale1000", "e1000"
+                            RbVmomi::VIM::VirtualE1000
+                        when "virtuale1000e", "e1000e"
+                            RbVmomi::VIM::VirtualE1000e
+                        when "virtualpcnet32", "pcnet32"
+                            RbVmomi::VIM::VirtualPCNet32
+                        when "virtualsriovethernetcard", "sriovethernetcard"
+                            RbVmomi::VIM::VirtualSriovEthernetCard
+                        when "virtualvmxnetm", "vmxnetm"
+                            RbVmomi::VIM::VirtualVmxnetm
+                        when "virtualvmxnet2", "vmnet2"
+                            RbVmomi::VIM::VirtualVmxnet2
+                        when "virtualvmxnet3", "vmxnet3"
+                            RbVmomi::VIM::VirtualVmxnet3
+                        else # If none matches, use VirtualE1000
+                            RbVmomi::VIM::VirtualE1000
+                   end
+
+        if network.class == RbVmomi::VIM::Network
+            backing = RbVmomi::VIM.VirtualEthernetCardNetworkBackingInfo(
+                        :deviceName => bridge,
+                        :network => network)
+        else
+            port    = RbVmomi::VIM::DistributedVirtualSwitchPortConnection(
+                        :switchUuid => 
+                                network.config.distributedVirtualSwitch.uuid,
+                        :portgroupKey => network.key)
+            backing = 
+              RbVmomi::VIM.VirtualEthernetCardDistributedVirtualPortBackingInfo(
+                 :port => port)
+        end
+
+        return {:operation => :add,
+                :device => nic_card.new(
+                            :key => 0, 
+                            :deviceInfo => {
+                                :label => "net" + card_num.to_s,
+                                :summary => bridge
+                            },
+                            :backing => backing,
+                            :addressType => mac ? 'manual' : 'generated',
+                            :macAddress  => mac
+                           )
+               }
+    end
+
     ########################################################################
     #  Clone a vCenter VM Template and leaves it powered on
     ########################################################################
@@ -784,7 +1189,7 @@ private
             !type.nil? && type.text.downcase == "vcenter"
         }
 
-        raise "Cannot find VCenter element in VM template." if template.nil?
+        raise "Cannot find vCenter element in VM template." if template.nil?
 
         uuid = template.elements["VM_TEMPLATE"]
 
@@ -813,10 +1218,28 @@ private
             :powerOn  => false,
             :template => false)
 
-        vm = vc_template.CloneVM_Task(
-            :folder => vc_template.parent,
-            :name   => "one-#{vmid}",
-            :spec   => clone_spec).wait_for_completion
+        begin
+            vm = vc_template.CloneVM_Task(
+                :folder => vc_template.parent,
+                :name   => "one-#{vmid}",
+                :spec   => clone_spec).wait_for_completion
+        rescue Exception => e
+
+            if !e.message.start_with?('DuplicateName')
+                raise "Cannot clone VM Template: #{e.message}"
+            end
+
+            vm = connection.find_vm("one-#{vmid}")
+
+            raise "Cannot clone VM Template" if vm.nil?
+
+            vm.Destroy_Task.wait_for_completion
+            
+            vm = vc_template.CloneVM_Task(
+                :folder => vc_template.parent,
+                :name   => "one-#{vmid}",
+                :spec   => clone_spec).wait_for_completion
+        end
 
         vm_uuid = vm.config.uuid
 
@@ -829,7 +1252,8 @@ private
             vnc_listen = vnc_listen.text
         end
 
-        config_array = []
+        config_array     = []
+        context_vnc_spec = {}
 
         if vnc_port
             config_array +=
@@ -840,10 +1264,10 @@ private
 
         if context
             # Remove <CONTEXT> (9) and </CONTEXT>\n (11)
-            context_text = ""
+            context_text = "# Context variables generated by OpenNebula\n"
             context.elements.each{|context_element|
-                context_text += context_element.name + "=" +
-                                context_element.text + "\n"
+                context_text += context_element.name + "='" +
+                                context_element.text.gsub("'", "\\'") + "'\n"
             }
             context_text = Base64.encode64(context_text.chop)
             config_array +=
@@ -852,7 +1276,27 @@ private
         end
 
         if config_array != []
-            spec_hash = {:extraConfig =>config_array}
+            context_vnc_spec = {:extraConfig =>config_array}
+        end
+
+        # Take care of the NIC section, build the reconfig hash
+        nics     = xml.root.get_elements("//TEMPLATE/NIC")
+        nic_spec = {}
+
+        if !nics.nil?
+            nic_array = []
+            nics.each{|nic|
+               mac    = nic.elements["MAC"].text
+               bridge = nic.elements["BRIDGE"].text
+               model  = nic.elements["MODEL"] ? nic.elements["MODEL"].text : nil
+               nic_array << calculate_addnic_spec(vm, mac, bridge, model)
+            }
+
+            nic_spec = {:deviceChange => nic_array}
+        end
+
+        if !context_vnc_spec.empty? or !nic_spec.empty?
+            spec_hash = context_vnc_spec.merge(nic_spec)
             spec      = RbVmomi::VIM.VirtualMachineConfigSpec(spec_hash)
             vm.ReconfigVM_Task(:spec => spec).wait_for_completion
         end
